package fr.wfn.artilleur;


import java.text.DecimalFormat;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.app.Activity;
import android.view.Menu;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;
import java.math.*;

public class MainActivity extends Activity implements SeekBar.OnSeekBarChangeListener, SensorEventListener  {
	
	// paramètres du GUI
	private TextView distanceTxtView ;
	private SeekBar  choixDistance;
	private float distance;
	
	private TextView altitudeTxtView ;
	private SeekBar  choixAltitude;
	private float altitude;
	
	private TextView angleMaxTxtView;
	private TextView angleMinTxtView;
	
	private  SensorManager mSensorManager;
    private  Sensor mAccelerometer;
    private DecimalFormat formateur ;
	private TextView textAcceleration ;
	
	//paramètres du modèle de calcul
	
	double x 		=	0.0; // distance cible en m
	double y 		=	0.0; // altitude cible en m
	double x0 		=	-0.023; // position axe rotation canon en m
	double y0 		=	0.053; // position axe rotation canon en m
	double l		=	0.083; // longueur du canon en m
	double a_deg	=	0.0; // angle de tir en degrées
	double a_rad	=	0.0; // angle de tir en radians
	double V0		= 	0.0; // vitesse initiale en m.s-1
	double g 		= 	9.81;// accélération de pesanteur en m.s-2
	
	double a 		=	1.0;
	double b 		=	1.0;
	double c 		=	1.0;
	double d 		=	1.0;
	
	
	private double calculerV0(){
		double angle45 = 45.0/180*Math.PI;
		return Math.sqrt(0.5*g*Math.pow(((x-x0)/Math.cos(angle45)-l),2)/
						(Math.tan(angle45)*(x-x0)+y0-y));
	}
	
	
	private double F(double x) {
	    return c*x*x+b*x+d+a*Math.sqrt(x*x-1);
	}

	private double dicho(double x1, double x2){
	    double x = -1;
	    double y1,y2;
	    while (Math.abs(x1-x2)>0.0000001){
	    	y1 = F(x1);
	    	y2 = F(x2);
	    	x = (x1 + x2) * 0.5;
	    	y = F(x);
	    	if (y1*y2 < 0){
	    		if (y1*y < 0){
	    			x1 = x1;
	    			x2 = x;
	    		}else{
	    			x1 = x;
	    			x2 = x2;
	    		}
	    	}else{
	    		x = -1;
	    		x1 = x;
	    		x2 = x;
	    	}
	    }
	    return x;
	}
	
	

	
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		// initialisation du GUI
		setContentView(R.layout.activity_main);
		distanceTxtView = (TextView)findViewById(R.id.textDistanceCible);
		//distanceTxtView.setText("***");
		choixDistance = (SeekBar)findViewById(R.id.seekBarDistanceCible);
		choixDistance.setOnSeekBarChangeListener(this);
		
		altitudeTxtView = (TextView)findViewById(R.id.textAltitudeCible);
		//altitudeTxtView.setText("***");
		choixAltitude = (SeekBar)findViewById(R.id.seekBarAltiCible);
		choixAltitude.setOnSeekBarChangeListener(this);
		
		angleMaxTxtView = (TextView)findViewById(R.id.TextAngleMax);
		//angleMaxTxtView.setText("°");
		angleMinTxtView = (TextView)findViewById(R.id.textAngleMin);
		//angleMinTxtView.setText("°");
		
		mSensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
        mAccelerometer = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        formateur = new DecimalFormat("##0.0");
        textAcceleration = (TextView)findViewById(R.id.textAngleSensor);
		//textAcceleration.setText("*****");
      
		// initialisation du modèle
		
	}

	
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	
	public void onProgressChanged(SeekBar barre, int arg1, boolean arg2) {
		if (barre.equals(choixDistance)){
			distanceTxtView.setText(Integer.toString(barre.getProgress())+" cm");
		}
		if (barre.equals(choixAltitude)){
			altitudeTxtView.setText(Integer.toString(barre.getProgress())+" cm");
		}
		
	}
	
	public void onStartTrackingTouch(SeekBar arg0) {	}
	
	public void onStopTrackingTouch(SeekBar arg0) {}
	
	public void calibrationVitesse(View view){
		x = (double)choixDistance.getProgress()*0.01;
		y = (double)choixAltitude.getProgress()*0.01;
		V0 = calculerV0();
	}
	
	public void calculAnglesTir(View view){
		x = (double)choixDistance.getProgress()*0.01;
		y = (double)choixAltitude.getProgress()*0.01;
		
		a = x - x0;
		b = l*g*(x - x0)/(V0*V0);
		c = -0.5*g/(V0*V0)*a*a;
		d = -(y-y0-l*l*g*0.5/(V0*V0));
		
		double a1 = Math.acos(1/dicho(1, Math.sqrt(2)))/Math.PI*180;
		if (a1 < 91){
			angleMaxTxtView.setText(formateur.format(a1)+" °");
		}else{
			angleMaxTxtView.setText("--- °");
		}
		a1 = Math.acos(1/dicho(Math.sqrt(2),1000000))/Math.PI*180;
		if (a1 < 91){
			angleMinTxtView.setText(formateur.format(a1)+" °");
		}else{
			angleMinTxtView.setText("--- °");
		}
	}
	
	protected void onResume() {
        super.onResume();
        mSensorManager.registerListener(this, mAccelerometer, SensorManager.SENSOR_DELAY_NORMAL);
    }

    protected void onPause() {
        super.onPause();
        mSensorManager.unregisterListener(this);
    }

	public void onAccuracyChanged(Sensor arg0, int arg1) {}
	public void onSensorChanged(SensorEvent event) {
		float ax = event.values[0];
		float ay = event.values[1];
		float az = event.values[2];
		double angle = Math.atan2(ay, az)/Math.PI*180;
		textAcceleration.setText(formateur.format(angle)+" °");
	}

}
